@@ -49,7 +49,7 @@ __author__ = 'nnorwitz@google.com (Neal Norwitz)'
 VISIBILITY_PUBLIC, VISIBILITY_PROTECTED, VISIBILITY_PRIVATE = list(range(3))
 
 FUNCTION_NONE = 0x00
-FUNCTION_CONST = 0x01
+FUNCTION_SPECIFIER = 0x01
 FUNCTION_VIRTUAL = 0x02
 FUNCTION_PURE_VIRTUAL = 0x04
 FUNCTION_CTOR = 0x08
@@ -494,7 +494,7 @@ class TypeConverter(object):
 
         # Handle default (initial) values properly.
         for i, t in enumerate(parts):
-            if t.name == '[':
+            if t.name == '[' and arrayBegin == 0:
                 arrayBegin = i
                 other_tokens.append(t)
             elif t.name == ']':
@@ -615,6 +615,8 @@ class TypeConverter(object):
                 pointer = reference = array = False
                 first_token = None
                 default = []
+            elif default:
+                default.append(s)
             elif s.name == '*':
                 pointer = True
             elif s.name == '&':
@@ -626,8 +628,6 @@ class TypeConverter(object):
             elif s.name == '=':
                 # Got a default value. Add any value (None) as a flag.
                 default.append(None)
-            elif default:
-                default.append(s)
             else:
                 type_modifiers.append(s)
         add_parameter()
@@ -674,16 +674,15 @@ class ASTBuilder(object):
         # Using a deque should be even better since we access from both sides.
         self.token_queue = []
         self.namespace_stack = namespace_stack[:]
+        self.define = set()
         self.quiet = quiet
         self.in_class = in_class
         self.in_class_name_only = None
         if in_class is not None:
             self.in_class_name_only = in_class.split('::')[-1].split('<')[0]
         self.visibility = visibility
-        self.current_token = None
         # Keep the state whether we are currently handling a typedef or not.
         self._handling_typedef = False
-
         self.converter = TypeConverter(self.namespace_stack)
 
     def handle_error(self, msg, token):
@@ -698,9 +697,6 @@ class ASTBuilder(object):
             if not token:
                 break
 
-            # Get the next token.
-            self.current_token = token
-
             # Dispatch on the next token type.
             if (
                 token.token_type == _INTERNAL_TOKEN and
@@ -834,13 +830,32 @@ class ASTBuilder(object):
                 # Remove "define".
                 name = name[6:].strip()
                 assert name
+                # Handle #define \<newline> MACRO.
+                if name.startswith('\\'):
+                    name = name[1:].strip()
                 value = ''
+                paren = 0
+
                 for i, c in enumerate(name):
-                    if c.isspace():
+                    if not paren and c.isspace():
                         value = name[i:].lstrip()
                         name = name[:i]
                         break
+                    if c == ')':
+                        value = name[i + 1:].lstrip()
+                        name = name[:paren]
+                        self.define.add(name)
+                        break
+                    if c == '(':
+                        paren = i
+                if value.startswith('\\'):
+                    value = value[1:]
                 return Define(token.start, token.end, name, value)
+            if name.startswith('undef'):
+                # Remove "undef".
+                name = name[5:].strip()
+                assert name
+                self.define.discard(name)
             if name.startswith('if') and name[2:3].isspace():
                 condition = name[3:].strip()
                 if condition.startswith('0') or condition.startswith('(0)'):
@@ -863,7 +878,12 @@ class ASTBuilder(object):
             elif last_token.name == ']':
                 count -= 1
             tokens.append(last_token)
-            last_token = self._get_next_token()
+            temp_token = self._get_next_token()
+            if temp_token.name == '(' and last_token.name in self.define:
+                # TODO: for now just ignore the tokens inside the parenthesis
+                list(self._get_parameters())
+                temp_token = self._get_next_token()
+            last_token = temp_token
         return tokens, last_token
 
     def _ignore_up_to(self, token_type, token):
@@ -1016,8 +1036,12 @@ class ASTBuilder(object):
             token = tokenize.Token(tokenize.SYNTAX, ';', 0, 0)
 
         while token.token_type == tokenize.NAME:
-            if token.name == 'const':
-                modifiers |= FUNCTION_CONST
+            if (
+                token.name == 'const' or
+                token.name == 'override' or
+                token.name == 'final'
+            ):
+                modifiers |= FUNCTION_SPECIFIER
                 token = self._get_next_token()
             elif token.name == '__attribute__':
                 # TODO(nnorwitz): handle more __attribute__ details.
@@ -1099,9 +1123,8 @@ class ASTBuilder(object):
             body = None
             if token.name == '=':
                 token = self._get_next_token()
-                assert_parse(token.token_type == tokenize.CONSTANT, token)
-                assert_parse(token.name == '0', token)
-                modifiers |= FUNCTION_PURE_VIRTUAL
+                if token.name == '0':
+                    modifiers |= FUNCTION_PURE_VIRTUAL
                 token = self._get_next_token()
 
             if token.name == '[':
@@ -1679,10 +1702,6 @@ class ASTBuilder(object):
 
     def handle_return(self):
         tokens = self._get_tokens_up_to(tokenize.SYNTAX, ';')
-        if not tokens:
-            return Return(self.current_token.start,
-                          self.current_token.end,
-                          None)
         return Return(tokens[0].start, tokens[0].end, tokens)
 
     def handle_goto(self):
