@@ -145,12 +145,13 @@ class WarningHunter(object):
             msg = "unable to find '{}'".format(filename)
             self._add_warning(msg, node)
         else:
-            builder = ast.builder_from_source(source, filename,
-                                              quiet=self.quiet)
             try:
+                builder = ast.builder_from_source(source, filename,
+                                                  quiet=self.quiet)
                 module = Module(filename,
                                 [_f for _f in builder.generate() if _f])
-            except ast.ParseError as error:
+            except (ast.ParseError,
+                    tokenize.TokenError) as error:
                 if not self.quiet:
                     print(
                         "Exception while processing '{}': {}".format(
@@ -287,7 +288,7 @@ class WarningHunter(object):
                 # exception should not happen...unless the code relies
                 # on another header for proper compilation.
                 # Store the use since we might really need to #include it.
-                if namespace and not None in namespace and not '::' in name:
+                if namespace and None not in namespace and '::' not in name:
                     name = '::'.join(namespace) + '::' + name
                 file_uses[name] = file_uses.get(name, 0) | USES_DECLARATION
                 return
